@@ -1,5 +1,6 @@
 import math
 import os
+from timeit import default_timer as timer
 
 import torch
 
@@ -66,11 +67,12 @@ def test_jacobian_at_different_loc_than_ee():
     J = chain.jacobian(th, locations=loc)
     assert torch.allclose(J, torch.cat((J_c1, J_c2)), atol=1e-7)
 
+
 def test_jacobian_y_joint_axis():
     chain = pk.build_serial_chain_from_urdf(open(os.path.join(TEST_DIR, "simple_y_arm.urdf")).read(), "eef")
     th = torch.tensor([0])
     J = chain.jacobian(th)
-    J_c3 = torch.tensor([ [ [0.], [0.], [-0.3], [0.], [1.], [0.] ] ])
+    J_c3 = torch.tensor([[[0.], [0.], [-0.3], [0.], [1.], [0.]]])
     assert torch.allclose(J, J_c3, atol=1e-7)
 
 
@@ -146,6 +148,50 @@ def test_jacobian_prismatic():
                                             [0., 0., 0.]]]))
 
 
+def test_comparison_to_autograd():
+    chain = pk.build_serial_chain_from_urdf(open(os.path.join(TEST_DIR, "kuka_iiwa.urdf")).read(),
+                                            "lbr_iiwa_link_7")
+    d = "cuda" if torch.cuda.is_available() else "cpu"
+    chain = chain.to(device=d)
+
+    def get_pt(th):
+        return chain.forward_kinematics(th).transform_points(
+            torch.zeros((1, 3), device=th.device, dtype=th.dtype)).squeeze(1)
+
+    # compare the time taken
+    N = 1000
+    ths = (torch.tensor([[0.0, -math.pi / 4.0, 0.0, math.pi / 2.0, 0.0, math.pi / 4.0, 0.0]], device=d),
+           torch.rand(N - 1, 7, device=d))
+    th = torch.cat(ths)
+
+    autograd_start = timer()
+    j1 = torch.autograd.functional.jacobian(get_pt, inputs=th, vectorize=True)
+    # get_pt will produce N x 3
+    # jacobian will compute the jacobian of the N x 3 points with respect to each of the N x DOF inputs
+    # so j1 is N x 3 x N x DOF (3 since it only considers the position change)
+    # however, we know the ith point only has a non-zero jacobian with the ith input
+    j1_ = j1[range(N), :, range(N)]
+    pk_start = timer()
+    j2 = chain.jacobian(th)
+    pk_end = timer()
+    # we can only compare the positional parts
+    assert torch.allclose(j1_, j2[:, :3], atol=1e-6)
+    print(f"for N={N} on {d} autograd:{(pk_start - autograd_start) * 1000}ms")
+    print(f"for N={N} on {d} pytorch-kinematics:{(pk_end - pk_start) * 1000}ms")
+    # if we have functools (for pytorch>=1.13.0 it comes with installing pytorch)
+    try:
+        import functorch
+        ft_start = timer()
+        grad_func = functorch.vmap(functorch.jacrev(get_pt))
+        j3 = grad_func(th).squeeze(1)
+        ft_end = timer()
+        assert torch.allclose(j1_, j3, atol=1e-6)
+        assert torch.allclose(j3, j2[:, :3], atol=1e-6)
+        print(f"for N={N} on {d} functorch:{(ft_end - ft_start) * 1000}ms")
+    except:
+        pass
+
+
 if __name__ == "__main__":
     test_correctness()
     test_parallel()
@@ -153,3 +199,4 @@ if __name__ == "__main__":
     test_gradient()
     test_jacobian_prismatic()
     test_jacobian_at_different_loc_than_ee()
+    test_comparison_to_autograd()
